
//const Tweener = imports.ui.tweener;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Main = imports.ui.main;
const Search = imports.ui.search;
const SearchDisplay = imports.ui.searchDisplay;
const IconGrid = imports.ui.iconGrid;
const Gio = imports.gi.Gio;
const Meta = imports.gi.Meta;
const Util = imports.misc.util;
const Config = imports.misc.config;
const Gtk = imports.gi.Gtk;
const Lang = imports.lang;
const GLib = imports.gi.GLib;

let text, button;
let educSearch;
let educInstall;
let searchDisplay;
let viewTab;
let tabIndex;

const MAX_SEARCH_RESULTS_ROWS = 10;
const ICON_SIZE = 40;
const SEARCH_XML = '/var/lib/le-search/search.xml'
const RESULT_DIR = GLib.get_home_dir() + '/.le-search'

GLib.spawn_command_line_sync('mkdir -p ' + RESULT_DIR);

/* ------------------------------------------------------------------------------------------------- */

function SearchResult(result)
{
    this._init(result);
};

SearchResult.prototype =
{
    _init: function(resultMeta)
    {

        this.actor = new St.Bin({ style_class: 'contact',
                                 reactive: true,
                                 can_focus: true,
                                 track_hover: true });

        let content = new St.BoxLayout( { style_class: 'contact-content',
            vertical: false});
        this.actor.set_child(content);

        let iconObj = Gio.app_info_get_default_for_type(resultMeta.type[0],null).get_icon();
        let icon = St.TextureCache.get_default().load_gicon(null, iconObj, ICON_SIZE);

        content.add(icon, { x_fill: true,
                            y_fill: false,
                            x_align: St.Align.START,
                            y_align: St.Align.MIDDLE });

        let result = new St.BoxLayout({ style_class: 'contact-details',
                                        vertical: true});

        content.add(result, { x_fill: false, y_fill: false, x_align: St.Align.START });
        //this.content.set_child(result);

        //this.actor.connect('clicked', Lang.bind(this, this.onClickedContent));

        let authorLabel = new St.Label({ text: resultMeta.author,
                                             style_class: 'author'});
        let titleLabel = new St.Label({ text: resultMeta.title,
                                             style_class: 'title' });
        let categoryLabel = new St.Label({ text: resultMeta.category,
                                             style_class: 'category' });
        /*let debugLabel = new St.Label({ text: resultMeta.type[0],
                                             style_class: 'debug' });*/
        result.add(titleLabel, { x_fill: false, x_align: St.Align.START });
        result.add(authorLabel, { x_fill: false, x_align: St.Align.START });
        result.add(categoryLabel, { x_fill: false, x_align: St.Align.START });
        //result.add(debugLabel, { x_fill: false, x_align: St.Align.START });

    }


};

/* ------------------------------------------------------------------------------------------------- */

function searchContent(terms, result, install)
{

    let xmlFile;
    let filters = terms.join(' ');
    if( install )
    {
        GLib.spawn_command_line_sync('parses-contents ' +
            SEARCH_XML + ' ' + RESULT_DIR + '/result-install.xml install "' +
            filters + '" ' + filters);
        xmlFile = Shell.get_file_contents_utf8_sync(RESULT_DIR + '/result-install.xml').replace(/^<\?xml version="1.0" encoding="UTF-8"\?>/, '');
    }
    else
    {
        GLib.spawn_command_line_sync('parses-contents ' +
            SEARCH_XML + ' ' + RESULT_DIR + '/result-search.xml search "' +
            filters + '" ' + filters);
        xmlFile = Shell.get_file_contents_utf8_sync(RESULT_DIR + '/result-search.xml').replace(/^<\?xml version="1.0" encoding="UTF-8"\?>/, '');
    }

    let results = new XML(xmlFile);

    for(var i in results)
    {
        result.push(
        {
            'title'     : results.result[i].@title.toString(),
            'author'    : results.result[i].@author.toString(),
            'category' : results.result[i].@category.toString(),
            'path' : results.result[i].@path.toString(),
            'package' : results.result[i].@package.toString(),
            'type' : Gio.content_type_guess(results.result[i].@path.toString(), null)
        });
        if(result.length > 5) return;
    }

    //return result;
}

function foundContent(content, results, term)
{
    for(var i in results)
    {
        if( results[i].title == content.@title &&
            results[i].author == content.@author &&
            results[i].category == content.@category )
            return;
    }
    if( content.@title.toString().toLowerCase().search(term) != -1 ||
        content.@author.toString().toLowerCase().search(term) != -1 ||
        content.@category.toString().toLowerCase().search(term) != -1 )
    {
        return;
    }
}

/* ------------------------------------------------------------------------------------------------- */

function EducSearch()
{
    this._init.apply(this, arguments);
};

EducSearch.prototype =
{
    __proto__: Search.SearchProvider.prototype,

    _init: function(title, install)
    {
        Search.SearchProvider.prototype._init.call(this, title);
        this.install = install;
    },


    getInitialResultSet: function(terms) {
        //let expr = terms.join('');
        let result = [];

        searchContent(terms, result, this.install);

        return result;
    },

    getSubsearchResultSet: function(prevResults, terms) {
        return this.getInitialResultSet(terms);
    },

    getResultMetas: function(ids) {
        let metas = [];
        for (let i = 0; i < ids.length; i++) {
            metas.push({ 'id': ids[i],
                         'name': ids[i].title,
                         'createIcon': function(size)
                                        {
                                            let iconObj = Gio.app_info_get_default_for_type(ids.type[0],null).get_icon();
                                            return St.TextureCache.get_default().load_gicon(null, iconObj, ICON_SIZE);
                                        } });
        }
        return metas;
    },

    activateResult: function(id)
    {
        if(this.install)
        {
            Util.spawn(['software-center', id.package]);
        }
        else
        {
            switch(id.type[0])
            {
                case 'text/html':
                    Util.spawn(['firefox', id.path]);
                    break;
                case 'video/mp4':
                    Util.spawn(['totem', id.path]);
                    break;
                case 'application/pdf':
                    Util.spawn(['evince', id.path]);
                    break;
            }
        }
    },

    createResultActor: function(resultMeta, terms) {
        let result = new SearchResult(resultMeta.id);
        return result.actor;
    },

    createResultContainerActor: function() {
        /*let grid = new IconGrid.IconGrid({ rowLimit: MAX_SEARCH_RESULTS_ROWS,
                                           xAlign: St.Align.START });*/
        let grid = new IconGrid.IconGrid({ xAlign: St.Align.START });
        grid.actor.style_class = 'contact-grid';

        let actor = new SearchDisplay.GridSearchResults(this, grid);
        return actor;
    },
}

/* ------------------------------------------------------------------------------------------------- */

function SubEduc (label, timerange, template, sorting, multi_select, subjects, expandable)
{
    this._init (label, timerange, template, sorting, multi_select, subjects, expandable);
}

SubEduc.prototype =
{
    _init: function (label, timerange, template, sorting, multi_select, subjects, expandable)
    {
        this._expandable = expandable;
        this._item_limit = 5;
        this._subjects = subjects;
        this._items = [];
        this._events = [];
        this._timerange = timerange;
        this._template = template;
        this._sorting = sorting;
        this._multi_select = multi_select;
        this._itemSpacing = 37; // "item-spacing" attribute
        this._rowSpacing = 8;  // "row-spacing" attribute11
        this._box = new St.BoxLayout({vertical: true });
        this._container = new Shell.GenericContainer ();
        this._container.connect ("style-changed", Lang.bind (this, this._styleChanged));
        this._container.connect ("allocate", Lang.bind (this, this._allocate));
        this._container.connect ("get-preferred-width", Lang.bind (this, this._getPreferredWidth));
        this._container.connect ("get-preferred-height", Lang.bind (this, this._getPreferredHeight));
        //this._container.add_actor(label.actor);
        this._header = new St.BoxLayout({ vertical: false, style_class: 'journal-heading-box'});
        this._box.add_actor (this._header, { x_align: St.Align.START, y_align: St.Align.START, expand: true, x_fill: true, y_fill: false });
        this._box.add_actor (this._container, { expand: true, x_fill: true, y_fill: true});
        this.actor = this._box;
        this._label = label;
        this._inserted_items = [];
        this._expanded = false;


        this.refresh();
    },

    refresh : function ()
    {
        this._items = []

        var widgets = this._container.get_children();
        for (var i = 0; i < widgets.length; i++)
        {
            this._container.remove_actor(widgets[i]);
        }
        var widgets = this._header.get_children();
        for (var i = 0; i < widgets.length; i++)
        {
            this._header.remove_actor(widgets[i]);
        }


        var heading = new HeadingItem(this._label);

        //this.appendItem (heading);

        this._header.add_actor(heading.actor, {x_align: St.Align.START, y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

        Zeitgeist.findEvents (this._timerange,            // time_range
                              [this._template],           // event_templates
                              Zeitgeist.StorageState.ANY, // storage_state - FIXME: should we use AVAILABLE instead?
                              0,                          // num_events - 0 for "as many as you can"
                              this._sorting, // result_type
                              Lang.bind (this, this._appendEvents));
    },

    _appendEvents: function(events)
    {
        //this._subjects = {} //FIXME: we cant just remove the subjects 
        this.actor.hide();
        this._events = events;
        var inserted = 0;
        this._inserted_items = [];
        //log ("got " + events.length + " events");
        if (!this._expandable)
            this._item_limit = 999;
        for (let i = 0; i < events.length; i++) {
            let e = events[i];
            let subject = e.subjects[0];
            let uri = subject.uri.replace('file://', '');
            uri = GLib.uri_unescape_string(uri, '');
            if (GLib.file_test(uri, GLib.FileTest.EXISTS) || subject.origin.indexOf("Telepathy") != -1) {
                if (this._subjects[uri] == undefined)
                    this._subjects[uri] = {"uri": uri, "journal": this, "item": null, "event": e};
                var allow_insertion = false;
                if (inserted >= this._item_limit) {
                    if (this._subjects[uri]["item"] == null) {
                        inserted = inserted +1;
                        this._inserted_items.push(e);
                    }
                    continue;
                }
                let item = new EventItem (e, this._multi_select, this);
                if (this._subjects[uri]["item"] == null) {
                    this._subjects[uri]["item"] = item;
                    allow_insertion = true;
                }
                else if (item._item_info.timestamp > this._subjects[uri]["event"].timestamp
                    || this._subjects[uri]["journal"] == this) {
                    //log("---> found " + uri + " in other container");
                    // FIXME: remove item from other journal
                    // this._subjects[uri]["timestamp"] = e.timestamp
                    allow_insertion = true;
                }
                if (allow_insertion) {
                    this._inserted_items.push(e);
                    inserted = inserted +1;
                    if (inserted <= this._item_limit) {
                        this.appendItem (item);
                        this.appendHSpace ();
                    }
                }
            }
        }
        if (inserted > 0)
            this.actor.show();

        //[arg, func] = SubjCategories[this._label] ;
        if (this._expandable)
        {
            if (!(this._label in SubjCategories) && inserted > this._item_limit){
                this._moreButton = new St.Button({ label: GLib.markup_escape_text (
                    "(Show " + (this._inserted_items.length - this._item_limit) + " items...)", -1),
                                         style_class: 'journal-heading',
                                         y_align: St.Align.START,
                                         x_align: St.Align.START,
                                         can_focus: true });
                this._header.add(this._moreButton, {x_align: St.Align.START, y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

                this._moreButton.connect('clicked', Lang.bind(this, function() {
                    this._toggleMore();
                }));
            }
            else if (this._label in SubjCategories) {
                this._moreButton = new St.Button({ 
                    label: GLib.markup_escape_text ("(Show More...)", -1),
                    style_class: 'journal-heading',
                    x_align: St.Align.START,
                    y_align: St.Align.START,
                    can_focus: true });
                this._header.add(this._moreButton, {x_align: St.Align.START, y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

                this._moreButton.connect('clicked', Lang.bind(this, function() {
                    [arg, func] = SubjCategories[this._label];
                func._selectCategory(arg);
                }));
            }
        }
    },

    _toggleMore: function ()
    {
        this._expanded = !this._expanded;
        if (this._expanded)
        {
            this._moreButton.set_label("(Show fewer items)");
            this._item_limit = 999;
        }
        else
        {
            this._item_limit = 5;
            this._moreButton.set_label("(+ " + (this._inserted_items.length - this._item_limit) + " items...)")
        }

        var widgets = this._container.get_children();
        this._items = []
        for (var i = 0; i < widgets.length; i++)
        {
            this._container.remove_actor(widgets[i]);
            widgets[i].destroy();
        }

        this._appendEvents(this._events);
    },

    _styleChanged: function ()
    {
        for (var key in this._containers)
        {
            let node = this._containers[key].get_theme_node ();

            this._itemSpacing = node.get_length ("item-spacing");
            this._rowSpacing = node.get_length ("row-spacing");

            this._containers[key].queue_relayout ();
        }
    },

    _allocate: function (actor, box, flags)
    {
        let width = box.x2 - box.x1;
        this._computeLayout (width, true, flags);
    },
    
    _getPreferredWidth: function (actor, forHeight, alloc)
    {
        alloc.min_size = 128; // FIXME: get the icon size from CSS
        alloc.natural_size = (48 + this._itemSpacing) * 4 - this._itemSpacing; // four horizontal icons and the spacing between them
    },

    _getPreferredHeight: function (actor, forWidth, alloc)
    {
        let height = this._computeLayout (forWidth, true, null);
        alloc.min_size = height;
        alloc.natural_size = height;
    },

    _computeLayout: function (available_width, do_allocation, allocate_flags)
    {
        let layout_state =  { newline_goal_column: 0,
                              x: 0,
                              y: 0,
                              row_height : 0,
                              layout_width: available_width
                            };

        let newline = Lang.bind (this, function ()
        {
            layout_state.x = layout_state.newline_goal_column;
            layout_state.y += layout_state.row_height + this._rowSpacing;
            layout_state.row_height = 0;
        });

        for (let i = 0; i < this._items.length; i++)
        {
            let item = this._items[i];
            let item_layout = { width: 0, height: 0 };

            if (item.type == "item")
            {
                if (!item.child)
                    throw new Error ("internal error - item.child must not be null");

                item_layout.width = item.child.actor.get_preferred_width (-1)[1]; // [0] is minimum width; [1] is natural width
                item_layout.height = item.child.actor.get_preferred_height (item_layout.width)[1];
            } else if (item.type == "newline") {
                newline ();
                continue;
            } else if (item.type == "hspace") {
                item_layout.width = this._itemSpacing;
            }

            if (layout_state.x + item_layout.width > layout_state.layout_width)
            {
                newline ();

                if (item.type == "hspace")
                    continue;
            }

            let box = new Clutter.ActorBox ();
            box.x1 = layout_state.x;
            box.y1 = layout_state.y;
            box.x2 = box.x1 + item_layout.width;
            box.y2 = box.y1 + item_layout.height;
            
            if (item.type == "item" && do_allocation && allocate_flags != null)
            {
                item.child.actor.allocate (box, allocate_flags);
            }

            layout_state.x += item_layout.width;
            if (item_layout.height > layout_state.row_height)
                layout_state.row_height = item_layout.height;
        }

        return layout_state.y + layout_state.row_height;
    },

    // We only expect items to have an item.actor field, which is a ClutterActor
    appendItem: function (item)
    {
        if (!item)
            throw new Error ("item must not be null");
        if (!item.actor)
            throw new Error ("Item must already contain an actor when added to the JournalLayout");
        let i = { type: "item",
                  child: item };
        this._items.push (i);
        this._container.add_actor (item.actor);
    },

    appendNewline: function ()
    {
        let i = { type: "newline" }
        this._items.push (i);
    },

    appendHSpace: function ()
    {
        let i = { type: "hspace" };
        this._items.push (i);
    },

    removeItem: function (item)
    {
        for (var i = 0; i < this._items.length; i++)
        {
            if (this._items[i].type == 'item' && this._items[i].child == item)
            {
                this._items.splice(i, i+1);
                log("-------------------------------------");
                break;
            }
        }
    }
}

function MultiSelect ()
{
        this._init ();
}

MultiSelect.prototype = {
        _init: function () {
                this._elements = [];
                this._multi_select = false;
        },

        select: function (source, item) {
                if (this._elements.length == 0)
                        this._multi_select = true;
                else
                        this._multi_select = false;

                if (this.isSelected (item) || this._multi_select) {
                        let e = { source : source,
                                          item: item,
                                          selected: true };
                        source.add_style_class_name('journal-item-selection');
                        this._elements.push(e);
                } else {
                        this.unselect (source, item);
                }
        },

        unselect: function (source, item) {
                source.remove_style_class_name('journal-item-selection');        
                for (let i = 0; i < this._elements.length; i++) {
                        if (this._elements[i].source == source) {
                                this._elements.splice(i, 1);
                                break;
                        }
                }
        },

        isSelected: function (item) {
                for (let i = 0; i < this._elements.length; i++) {
                        let e = this._elements[i];
                        if ((e.item == item) && e.selected) {
                                return false;
                        }
                }
                return true;
        },

        destroy: function() {
          let elements = this._elements;
          for (let i = 0; i < elements.length; i++) {
                  let e = elements[i];
                  e.source.remove_style_class_name('journal-item-selection');        
          }
          this._elements = [];
        },

        querySelections: function () {
                return this._elements;
        }
}

function EducLayout()
{
    this._init();
}

EducLayout.prototype =
{
    _init: function()
    {
        this._items = [];

        this._itemSpacing = 0;
        this._rowSpacing = 0;

        this._box = new St.BoxLayout({name: 'searchResultsContent', vertical: true});
        this.actor = this._box;

        this._containers = {};
    },


    _setFilterViews: function (format, category)
    {
        this.clear()
        //var end = new Date().getTime();
        let template = category.event_template;
        let offset = category.time_range;
        let sorting = category.sorting;

        let multi_select = new MultiSelect ();
        let uri_map = {}

        if (format == false)
        {
            if (offset > 0)
                start = end - offset
            else
                start = 0

            template.subjects[0].interpretation = Semantic.NFO_DOCUMENT;
            this._containers["SchoolTV"] = new SubEduc ("TV Escola",
                [start, end], eval(uneval(template)), sorting, multi_select, uri_map, true);
            this._box.add_actor (this._containers["SchoolTV"].actor, { y_align: St.Align.START, expand: true, x_fill: true, y_fill: true });

        }
    },
    clear: function ()
    {
        this._items = [];
        for (var key in this._containers)
        {
            this._containers[key].actor.destroy_all_children();
            this._containers[key].actor.destroy();
            this.actor.destroy_all_children();
        }
    }
}

/* ------------------------------------------------------------------------------------------------- */

function EducDisplay()
{
    this._init();
}

EducDisplay.prototype =
{
    _init: function ()
    {
        this.box = new St.BoxLayout({style_class: 'all-app' });
        this._scroll_view = new St.ScrollView({ x_fill: false,
            y_fill: false,
            style_class: 'vfade' });

        this._scroll_view.set_policy (Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
        this._scroll_view.connect ("notify::mapped", Lang.bind (this, this._scrollViewMapCb));

        this._layout = new EducLayout ();

        this._filters = new St.BoxLayout({ vertical: true, reactive: true });
        this._scroll_view.add_actor(this._layout.actor, { expand: true, y_fill: true,y_align: St.Align.START, x_fill: true });

        this._categoryScroll = new St.ScrollView({ x_fill: false,
            y_fill: false,
            style_class: 'vfade' });
        this._categoryScroll.add_actor(this._filters);

        this.box.add(this._scroll_view, { expand: true, x_fill: true, y_fill: true });
        this.box.add_actor(this._categoryScroll, { expand: false, y_fill: false, y_align: St.Align.START  });

        this.actor = this.box;
        this._sections = [];
        this._setFilters();
        this._selectCategory(1);
        //this._filters.connect('scroll-event', Lang.bind(this, this._scrollFilter))
    },

    _scrollFilter: function(actor, event)
    {
        let direction = event.get_scroll_direction();
        if (direction == Clutter.ScrollDirection.UP)
            this._selectCategory(Math.max(this._currentCategory - 1, -1))
        else if (direction == Clutter.ScrollDirection.DOWN)
            this._selectCategory(Math.min(this._currentCategory + 1, this._sections.length - 1));
    },

    _selectCategory: function(num)
    {
        this._currentCategory = num;

        for (let i = 0; i < this._sections.length; i++)
        {
            if (i == num)
                this._sections[i].add_style_pseudo_class('selected');
            else
                this._sections[i].remove_style_pseudo_class('selected');
        }


        //this._layout._setFilterViews((num > 2), this._categories[num]);
    },

    _addCategory: function (category, linkable)
    {
        let button = new St.Button({ label: GLib.markup_escape_text (category.title, -1),
            style_class: 'app-filter',
            x_align: St.Align.START,
            can_focus: true });
        this._filters.add(button, { expand: false, x_fill: true, y_fill: false });

        this._sections[this._counter] = button;

        var x = this._counter;
        button.connect('clicked', Lang.bind(this, function() {
            this._selectCategory(x);
        }));
        this._categories.push(category);
        this._counter = this._counter + 1;

        if (linkable == true)
            SubjCategories[category.title] = [x, this];
    },

    _scrollViewMapCb: function (actor)
    {
        if (this._scroll_view.mapped)
            this._reload ();
    },

    _reload: function ()
    {
        this._selectCategory(this._currentCategory)
    },

    _setFilters: function ()
    {
        this._counter = 0;
        this._categories = [];
        this._addCategory(new PublicDomain());
        this._addCategory(new SchoolTV());
        this._addCategory(new TeacherPortal());

        var space = new St.Label ({ text: "",
                                     style_class: 'app-filter' });
        this._filters.add(space, { expand: false, x_fill: true, y_fill: false });

        this._addCategory(new TextFormat());
        this._addCategory(new SoundFormat());
        this._addCategory(new PictureFormat());

        var space = new St.Label ({ text: "",
            style_class: 'app-filter' });
        this._filters.add(space, { expand: false, x_fill: true, y_fill: false });

        //this._addCategory(new DocumentsCategory(), true);
    },
}

/* ------------------------------------------------------------------------------------------------- */

function CategoryInterface(title)
{
    this._init(title);
}

CategoryInterface.prototype =
{
    _init: function(title)
    {
        this.title = title;
        this.func = null;
        this.subCategories = [];
        this.event_template = null;
        this.time_range = null;
        this.sorting = 2;
    },
}

function PublicDomain()
{
    this._init();
}

PublicDomain. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Dom\xEDnio P\xFAblico"));
    }
}


function SchoolTV()
{
    this._init();
}

SchoolTV. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("TV Escola"));
    }
}

function TeacherPortal()
{
    this._init();
}

TeacherPortal. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Portal do Professor"));
    }
}

function TextFormat()
{
    this._init();
}

TextFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Texto"));
    }
}

function SoundFormat()
{
    this._init();
}

SoundFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Som"));
    }
}
function PictureFormat()
{
    this._init();
}

PictureFormat. prototype =
{
    __proto__: CategoryInterface.prototype,
    _init: function()
    {
        CategoryInterface.prototype._init.call(this, _("Imagem"));
    }
}
/* ------------------------------------------------------------------------------------------------- */

function init()
{
    imports.gettext.bindtextdomain('gnome-shell-extensions', Config.LOCALEDIR);
    educSearch = new EducSearch('CONTE\xDADOS EDUCACIONAIS', false);
    educInstall = new EducSearch('INSTALAR CONTE\xDADOS EDUCACIONAIS RELACIONADOS', true);
    searchDisplay = new EducDisplay();
};

function enable() {
    Main.overview.addSearchProvider(educSearch);
    Main.overview.addSearchProvider(educInstall);

    Main.overview._viewSelector.addViewTab('eduContents', _("Buscar Conte\xFAdos Educacionais"), searchDisplay.actor, 'system-run');
    viewTab = Main.overview._viewSelector._tabs[Main.overview._viewSelector._tabs.length-1];
    tabIndex = Main.overview._viewSelector._tabs.length - 1;
};

function disable() {
    Main.overview.removeSearchProvider(educSearch);
    Main.overview.removeSearchProvider(educInstall);

    Main.overview._viewSelector._tabBox.remove_actor(viewTab.title);
    Main.overview._viewSelector._pageArea.remove_actor(viewTab.page);
    Main.overview._viewSelector._tabs.splice(tabIndex, tabIndex);
    searchDisplay.actor.destroy();
    searchDisplay = null;
};
